

#include <qlabel.h>
#include <kmessagebox.h>
#include <kconfig.h>
#include <kconfigdata.h>

#include <qdir.h>
#include <qfile.h>

#include "khdapsmonwidget.h"
#include "khdapsmon.h"
#include "sysfsparse.h"

khdapsmonWidget::khdapsmonWidget(QWidget* parent, const char* name, WFlags fl)
        : khdapsmonWidgetBase(parent,name,fl)
        , sysFsParser1( SysFsParse() )
{

     timer = new QTimer(this);
     connect(timer,SIGNAL(timeout()),this,SLOT(updateDialog()));
     timer->start(500);

     deviceList->insertStringList(initializeDriveList(), 0);
     updateConfig();
     connect(deviceList,SIGNAL(activated(int)),this,SLOT(updateConfig()));
}

khdapsmonWidget::~khdapsmonWidget()
{

     timer->stop();
     delete timer;
     timer = NULL;

     updateConfig();
}

void khdapsmonWidget::updateDialog() {

QString device = KGlobal::config()->readEntry("Device","hda");

    sysFsParser1.isQueueProtected(device) ? queueProtected->setText("yes") : queueProtected->setText("no");

    accelPos->setText(sysFsParser1.returnAccelCoords());

    sysFsParser1.periphrialActivity();

    sysFsParser1.keyb ? keybAct->setText("yes") : keybAct->setText("no");

    sysFsParser1.mous ? mouseAct->setText("yes") : mouseAct->setText("no");
}

QStringList khdapsmonWidget::initializeDriveList() {

    // I assume we would be able to monitor sdX disks (serial ATA?)
    // someone please let me know if this is not the case...
    QDir *sysBlock = new QDir("/sys/block","[hs]d*",QDir::IgnoreCase,QDir::Dirs);

    // sanity check
    if (!sysBlock->exists()) {
	KMessageBox::error(this,"/sys/block does not exist. Are you running Linux 2.6? Drive list will be empty.","Error",KMessageBox::Notify);
	return NULL;
    }

    QStringList driveList = sysBlock->entryList();
    // run through each directory and look for a "queue/protect" entry
    unsigned int i;
    for (i=0;i < driveList.count();i++) {
	if(!QFile("/sys/block/" + driveList[i] + "/queue/protect").exists())
		driveList.remove(driveList.at(i));
    }

    // final sanity check
    if (driveList.count() < 0) {
	KMessageBox::error(this,"No protectable drives were found. Perhaps you haven't patched your kernel?","Error",KMessageBox::Notify);
        // no point in returning null, AFAIK
    }

    return driveList;
}

void khdapsmonWidget::updateConfig() {
QString device = deviceList->currentText();
	KGlobal::config()->writeEntry("Device",device,TRUE,FALSE,FALSE);
	KGlobal::config()->sync();
}

#include "khdapsmonwidget.moc"

